@@ -70,9 +70,9 @@
 """
 
 
+import logging
 import ogAST
 
-import logging
 LOG = logging.getLogger(__name__)
 
 # reference to the ASN.1 Data view and to the process variables
@@ -101,19 +101,46 @@ def find_basic_type(a_type):
     return basic_type
 
 
-def get_type_of_parent(identifier):
-    ''' Return the type of a "parent" construct (a!b!c)=>return type of b '''
+def path_type(path):
+    '''
+        Return the type of a path construct
+        Input: path a!b!c in the form [a, b, c]
+        Output: parent_kind, parent_typename (strings)
+        Used for Ada to know about CHOICE types
+    '''
+    # NOTE: all this code is duplicated from the find_type function
+    # in ogParser - should be reused
+    if not path or not path[0].lower() in VARIABLES:
+        return None, None
     kind = ''
-    name = ''
-    if len(identifier) > 1:
-        if identifier[0] in VARIABLES:
-            name = VARIABLES[identifier[0]].replace('_', '-')
-            current_type = TYPES[name]['type']
-            for index in range(1, len(identifier)):
-                # TODO find type...
-                LOG.warning('** INCOMPLETE FEATURE ** ')
-            kind = current_type['Kind'].replace('-', '_')
-    return kind, name
+    asn1_name = VARIABLES[path[0].lower()].replace('_', '-')
+    # Get ASN.1 type of the first element
+    current = TYPES[asn1_name]['type']
+    if len(path) > 1:
+        for elem in path[1:]:
+            current = find_basic_type(current)
+            if 'procParams' in elem:
+                # Discard operator parameters: they do not change the type
+                continue
+            # Sequence, Choice (case insensitive)
+            if current['Kind'] in ('SequenceType', 'ChoiceType'):
+                elem_asn1 = elem.replace('_', '-').lower()
+                type_idx, = (c for c in current['Children']
+                                    if c.lower() == elem_asn1)
+                current = current['Children'][type_idx]['type']
+            # Sequence of
+            elif current['Kind'] == 'SequenceOfType':
+                current = current['type']
+            elif current['Kind'] == 'EnumeratedType':
+                pass
+            else:
+                raise TypeError('Expression ' + '.'.join(path) +
+                                ' does not resolve')
+        asn1_name = current['ReferencedTypeName'].replace('-', '_')
+    kind = find_basic_type(current).get('Kind').replace('-', '_')
+    LOG.debug('[Ada Generator] Type of path ' + '!'.join(path) +
+             ' is ' + asn1_name + ' (' + kind + ')')
+    return kind, asn1_name
 
 
 def traceability(symbol):
@@ -132,9 +159,13 @@ def write_statement(param, newline):
     local = []
     basic_type = find_basic_type(param.exprType) or {}
     type_kind = basic_type.get('Kind')
-    if type_kind == 'StringType':
-        # Raw string
-        string = '"' + param.inputString[1:-1] + '"'
+    if type_kind.endswith('StringType'):
+        if param.var.kind == 'stringLiteral':
+            # Raw string
+            string = '"' + param.var.stringLiteral[1:-1] + '"'
+        else:
+            # XXX Cannot print an octet string like that...
+            code, string, local = decipher_expression(param)
     elif type_kind in ('IntegerType', 'RealType', 'BooleanType'):
         code, string, local = decipher_expression(param)
         if type_kind == 'IntegerType':
@@ -181,6 +212,30 @@ def output_statement(output):
                 code.extend(stmts)
                 local_decl.extend(local)
             continue
+        elif signal_name.lower() == 'reset_timer':
+            # built-in operator for resetting timers. param = timer name
+            param, = out['params']
+            p_code, p_id, p_local = decipher_expression(param)
+            code.extend(p_code)
+            local_decl.extend(p_local)
+            code.append('RESET_{};'.format(p_id))
+            continue
+        elif signal_name.lower() == 'set_timer':
+            # built-in operator for setting a timer: SET(1000, timer_name)
+            timer_value, timer_id = out['params']
+            t_code, t_val, t_local = decipher_expression(timer_value)
+            p_code, p_id, p_local = decipher_expression(timer_id)
+            code.extend(t_code)
+            code.extend(p_code)
+            local_decl.extend(t_local)
+            local_decl.extend(p_local)
+            # Use a temporary variable to store the timer value
+            tmp_id = 'tmp' + str(out['tmpVars'][0])
+            local_decl.append('{} : aliased asn1SccT_Uint32;'.format(tmp_id))
+            code.append('{tmp} := {val};'.format(tmp=tmp_id, val=t_val))
+            code.append("SET_{timer}({value}'access);"
+                                             .format(timer=p_id, value=tmp_id))
+            continue
         proc, out_sig = None, None
         try:
             out_sig, = [sig for sig in OUT_SIGNALS
@@ -194,7 +249,7 @@ def output_statement(output):
                 # Not external? Must be an inner procedure then.
                 # otherwise the parser would have barked
                 proc, = [sig for sig in INNER_PROCEDURES
-                         if sig.inputString.lower() == signal_name.lower()]
+                     if sig.inputString.lower() == signal_name.lower()]
         if out_sig:
             list_of_params = []
             for idx, param in enumerate(out.get('params') or []):
@@ -214,7 +269,8 @@ def output_statement(output):
                 if param_direction == 'in':
                     tmp_id = out['tmpVars'][idx]
                     local_decl.append('tmp{idx} : aliased asn1Scc{oType};'
-                                      .format(idx=tmp_id, oType=param_type))
+                                      .format(idx=tmp_id,
+                                          oType=param_type.replace('-', '_')))
                     code.append('tmp{idx} := {p_id};'.format(
                         idx=tmp_id, p_id=p_id))
                     list_of_params.append("tmp{idx}'access".
@@ -283,17 +339,21 @@ def task_statement(task):
 
 def decipher_primary_id(primaryId):
     '''
-    Return the Ada string of a PrimaryId element list
+        Return the Ada string of a PrimaryId element list (path)
+        cases: a => 'l_a' (reference to a variable)
+        a_timer => 'a_timer'  (reference to a timer)
+        a!b => a.b (field of a structure)
+        a!b if a is a CHOICE => TypeOfa_b_get(a)
+        a(Expression) => a(ExpressionSolver) (array index)
+        Expression can be complex (if-then-else-fi..)
     '''
     ada_string = ''
     stmts = []
     local_decl = []
-    # cases: a => 'a' (reference to a variable)
-    # a!b => a.b (field of a structure)
-    # a!b if a is a CHOICE => TypeOfa_b_get(a)
-    # a(Expression) => a(ExpressionSolver) (array index)
-    # Expression can be complex (if-then-else-fi..)
-    sep = 'l_'
+   
+    # If first element is not a variable (can be a timer) do not add prefix
+    sep = 'l_' if primaryId[0].lower() in VARIABLES else ''
+    
     sub_id = []
     for pr_id in primaryId:
         if type(pr_id) is not dict:
@@ -304,19 +364,48 @@ def decipher_primary_id(primaryId):
                 special_op = 'ChoiceKind'
                 continue
             special_op = ''
+            parent_kind, parent_typename = path_type(sub_id)
             sub_id.append(pr_id)
-            parent_kind, parent_typename = get_type_of_parent(sub_id)
             if parent_kind == 'ChoiceType':
                 ada_string = ('asn1Scc{typename}_{p_id}_get({ada_string})'
-                        .format(typename=parent_typename, p_id=pr_id,
-                        ada_string=ada_string))
+                        .format(typename=parent_typename.replace('-', '_'),
+                                p_id=pr_id, ada_string=ada_string))
             else:
                 ada_string += sep + pr_id
         else:
-            # index is a LIST - only taking 1st elem.
-            if 'index' in pr_id:
-                if len(pr_id['index']) > 1:
-                    LOG.warning('Multiple index not supported')
+            if 'substring' in pr_id:
+                # substring: two parameters (range)
+                r1_stmts, r1_string, r1_local = decipher_expression(
+                        pr_id['substring'][0])
+                r2_stmts, r2_string, r2_local = decipher_expression(
+                        pr_id['substring'][1])
+                # should we add 1 in case of numerical values? (see index)
+                ada_string += '.Data({r1}..{r2})'.format(
+                                                r1=r1_string, r2=r2_string)
+                stmts.extend(r1_stmts)
+                stmts.extend(r2_stmts)
+                local_decl.extend(r1_local)
+                local_decl.extend(r2_local)               
+                _, parent_typename = path_type(sub_id) 
+                local_decl.append('tmp{idx} : aliased asn1Scc{parent_type};'
+                          .format(idx=pr_id['tmpVar'],
+                              parent_type=parent_typename.replace('-', '_')))
+                  # XXX types with fixed length: substrings will not work
+                if unicode.isnumeric(r1_string) and unicode.isnumeric(
+                                                                r2_string):
+                    length = eval(r2_string + '-' + r1_string + '+1')
+                else:
+                    length = ('{r2} - {r1} +1'
+                        .format(r2=r2_string, r1=r1_string))
+                stmts.append('tmp{idx}.Length := {length};'
+                        .format(idx=pr_id['tmpVar'], length=length))
+
+                stmts.append('tmp{idx}.Data(1..{length}) := {data};'
+                        .format(idx=pr_id['tmpVar'],
+                            length=length, data=ada_string))
+                ada_string = 'tmp{idx}'.format(idx=pr_id['tmpVar'])
+            elif 'index' in pr_id:
+                # index is a list but it can have only one element
                 idx_stmts, idx_string, local_var = decipher_expression(
                         pr_id['index'][0])
                 if unicode.isnumeric(idx_string):
@@ -410,7 +499,7 @@ def decipher_expression(expr):
         local_decl.extend(left_local)
         local_decl.extend(right_local)
     elif expr.kind == 'append':
-        # Append item to a list - Octet string (TODO: sequence of)
+        # Append item to a list - Octet string and SEQUENCE OF
         basic_type_expr = find_basic_type(expr.exprType)
         # We can do a length check if both strings are literals
         if(expr.right.kind == 'primary'
@@ -434,17 +523,19 @@ def decipher_expression(expr):
         ada_string = 'tmp{}'.format(expr.tmpVar)
         local_decl.append('{tmp} : aliased asn1Scc{eType};'.format(
                         tmp=ada_string,
-                        eType=expr.exprType['ReferencedTypeName']))
+                        eType=expr.exprType['ReferencedTypeName']
+                        .replace('-', '_')))
 
         # If right or left is raw, declare a temporary variable for it, too
         for sexp, sid in zip((expr.right, expr.left), (right_str, left_str)):
             if sexp.is_raw():
                 local_decl.append('tmp{idx} : aliased asn1Scc{eType};'.format(
-                        idx=sexp.var.tmpVar,
-                        eType=sexp.exprType['ReferencedTypeName']))
+                        idx=sexp.tmpVar,
+                        eType=sexp.exprType['ReferencedTypeName']
+                        .replace('-', '_')))
                 stmts.append('tmp{idx} := {s_id};'.format(
-                    idx=sexp.var.tmpVar, s_id=sid))
-                sexp.sid = 'tmp' + str(sexp.var.tmpVar)
+                    idx=sexp.tmpVar, s_id=sid))
+                sexp.sid = 'tmp' + str(sexp.tmpVar)
                 # Length of raw string - update for sequence of
                 if sexp.var.kind == 'stringLiteral':
                     sexp.slen = len(sexp.var.stringLiteral[1:-1])
@@ -483,12 +574,18 @@ def decipher_expression(expr):
         local_decl.extend(left_local)
         local_decl.extend(right_local)
         stmts.append("in_loop_{}:".format(ada_string))
-        stmts.append("for elem in {}.Data'Range loop".format(left_str))
+        left_type = find_basic_type(expr.left.exprType)
+        if left_type['Min'] != left_type['Max']:
+            stmts.append("for elem in 1..Integer({}.Length) loop"
+                         .format(left_str))
+        else:
+            stmts.append("for elem in {}.Data'Range loop".format(left_str))
         stmts.append("if {container}.Data(elem) = {pattern} then".format
                 (container=left_str, pattern=right_str))
         stmts.append("{} := True;".format(ada_string))
         stmts.append("end if;")
-        stmts.append("exit in_loop_{tmp} when {tmp} = True;".format(tmp=ada_string))
+        stmts.append("exit in_loop_{tmp} when {tmp} = True;"
+                      .format(tmp=ada_string))
         stmts.append("end loop in_loop_{};".format(ada_string))
 
     elif expr.kind == 'rem':
@@ -507,12 +604,13 @@ def decipher_primary(primary):
     if primary.kind == 'primaryId':
         stmts, ada_string, local_decl = decipher_primary_id(primary.primaryId)
     elif primary.kind == 'enumeratedValue':
-        # (note: in C we would need to read EnumID in the type definition)
-        ada_string = 'asn1Scc{enumVal}'.format(enumVal=primary.primaryId[0])
+        enumerant = primary.primaryId[0]
+        basic = find_basic_type(primary.primType)
+        ada_string = ('asn1Scc' 
+                      + basic['EnumValues'][enumerant]['EnumID'])
     elif primary.kind == 'choiceDeterminant':
         enumerant = primary.primaryId[0]
         ada_string = primary.primType['EnumValues'][enumerant]['EnumID']
-        #ada_string = '{choice}_PRESENT'.format(choice=primary.primaryId[0])
     elif primary.kind in ('numericalValue', 'booleanValue'):
         ada_string = primary.primaryId[0]
     elif primary.kind == 'sequence':
@@ -526,7 +624,7 @@ def decipher_primary(primary):
                 primary.choiceItem, primary.primType)
     elif primary.kind == 'emptyString':
         ada_string = 'asn1Scc{typeRef}_Init'.format(
-              typeRef=primary.primType['ReferencedTypeName'].replace('-', '_'))
+             typeRef=primary.primType['ReferencedTypeName'].replace('-', '_'))
     elif primary.kind == 'stringLiteral':
         basic_type = find_basic_type(primary.primType)
         if basic_type['Kind'].endswith('StringType'):
@@ -548,6 +646,8 @@ def decipher_primary(primary):
         stmts, ada_string, local_decl = decipher_expression(primary.expr)
     elif primary.kind == 'mantissaBaseExpFloat':
         pass
+    elif primary.kind == 'constant':
+        ada_string = primary.primaryId[0]
     else:
         raise ValueError('unsupported primary kind: ' + primary.kind)
     return stmts, ada_string, local_decl
@@ -586,6 +686,8 @@ def decipher_if_then_else(ifThenElse, resType):
 def decipher_sequence(seq, seqType):
     ''' Return Ada string for an ASN.1 SEQUENCE '''
     LOG.debug('Sequence: ' + str(seq) + str(seqType))
+    seq_ty = seqType.get('ReferencedTypeName', seqType)
+
     ada_string = 'asn1Scc{seqType}\'('.format(
             seqType=seqType['ReferencedTypeName'].replace('-', '_'))
     stmts = []
@@ -680,7 +782,7 @@ def decision_statement(dec):
     # for ASN.1 types, declare a local variable
     # to hold the evaluation of the question
     if not basic:
-        local_decl.append('tmp{idx} : asn1Scc{actType};'.format(
+        local_decl.append('tmp{idx} : aliased asn1Scc{actType};'.format(
             idx=dec.tmpVar, actType=actual_type))
     q_stmts, q_str, q_decl = decipher_expression(dec.question)
     # Add code-to-model traceability
@@ -869,17 +971,11 @@ def generate(process):
     VARIABLES.update(process.variables)
     global TYPES
     TYPES = process.dataview
-    # Ada is case insensitive => lower cases
-    #OUT_SIGNALS.update({sig.lower(): defn for sig, defn in
-    #        process.outputSignals.viewitems()})
     del OUT_SIGNALS[:]
     del PROCEDURES[:]
     del INNER_PROCEDURES[:]
     OUT_SIGNALS.extend(process.output_signals)
     PROCEDURES.extend(process.procedures)
-
-    #PROCEDURES.update({p.lower(): defn for p, defn in
-    #        process.procedures.viewitems()})
     INNER_PROCEDURES.extend(process.content.inner_procedures)
 
     LOG.info('Generating Ada code for process ' + str(process_name))
@@ -888,7 +984,7 @@ def generate(process):
     process_level_decl = []
     for var_name, var_type in VARIABLES.iteritems():
         process_level_decl.append('l_{n} : aliased asn1Scc{t};'.format(
-            n=var_name, t=var_type))
+            n=var_name, t=var_type.replace('-','_')))
 
     # Add the process states list to the process-level variables
     states_decl = 'type states is ('
@@ -906,6 +1002,8 @@ def generate(process):
     mapping = {}
     # Generate the code for the transitions in a mapping input-state
     input_signals = [sig['name'] for sig in process.input_signals]
+    # Add timers to the mapping
+    input_signals.extend(process.timers)
     for input_signal in input_signals:
         mapping[input_signal] = {}
         for state_name, input_symbols in process.mapping.viewitems():
@@ -961,9 +1059,10 @@ package {process_name} is'''.format(process_name=process_name,
     # Add the code of the procedures definitions
     taste_template.extend(inner_procedures_code)
 
-    # Generate the code for each input signal (provided interface)
-    for signal in process.input_signals:
-        if signal['name'] == 'START':
+    # Generate the code for each input signal (provided interface) and timers
+    for signal in process.input_signals + [
+                        {'name': timer.lower() for timer in process.timers}]:
+        if signal.get('name', 'START') == 'START':
             continue
         pi_header = 'procedure {sig_name}'.format(sig_name=signal['name'])
         param_name = signal.get('param_name') or 'MISSING_PARAM_NAME'
