@@ -67,8 +67,6 @@ from PySide.QtGui import(QGraphicsTextItem, QGraphicsPathItem,
 from PySide.QtUiTools import QUiLoader
 
 import undoCommands
-
-# Remove the following line when the Comment symbol is moved to sdlSymbols.py
 import ogAST
 import ogParser
 
@@ -405,8 +403,6 @@ class EditableText(QGraphicsTextItem, object):
         # Set width to auto-expand, and disables alignment, while editing:
         self.setTextWidth(-1)
         if not self.editing:
-            #print 'edit text macro start'
-            #self.scene().undo_stack.beginMacro('Edit text')
             self.oldText = str(self)
             self.oldSize = self.parentItem().boundingRect()
             self.editing = True
@@ -489,10 +485,6 @@ class Connection(QGraphicsPathItem, object):
         else:
             self.end_point = QPointF()
             self.end_point.setY(self.child.pos().y())
-            # FIXME - why is the pos of the child above the connection point
-            # when loading?! MP: fixed
-        #   self.end_point.setY(
-        #           max(self.child.pos().y(), self.start_point.y()))
         # Move to start point and draw the connection
         new_shape.moveTo(self.start_point)
         if not self.child_is_a_connection_point and not isinstance(
@@ -593,9 +585,6 @@ class Symbol(QObject, QGraphicsPathItem, object):
         self._terminal_symbol = False
         # and default text alignment within a textbox
         self.text_alignment = Qt.AlignLeft
-        #self.setFlags(
-        #        QGraphicsItem.ItemIsMovable)
-        #        | QGraphicsItem.ItemIsSelectable)
         # Activate cache mode to boost rendering by calling paint less often
         self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)
         # Apply symbol default mouse cursor
@@ -607,6 +596,8 @@ class Symbol(QObject, QGraphicsPathItem, object):
         self.coord = self.pos()
         self.height = 0
         self.old_rect = self.boundingRect()
+        # List of visible connection points (that can move)
+        self.movable_points = []
 
     def _pos(self):
         ''' Qt Property that can be used in animations '''
@@ -635,6 +626,37 @@ class Symbol(QObject, QGraphicsPathItem, object):
         #LOG.debug(str(followers))
         return followers
 
+    def closest_connection_point(self, coord):
+        '''
+           Given a position (QPointF), expected in this symbol's
+           coordinates, return the closest connection
+           point on the current symbol.
+           By default, return the RELATIVE POSITION of a point on the
+           bounding rectangle.
+           
+        '''
+        width = self.boundingRect().width()
+        height = self.boundingRect().height()
+        top, bottom = self.y(), self.y() + height
+        left, right = self.x(), self.x() + width
+        x, y = left + coord.x(), top + coord.y()
+        if (x > right or x < left) and (y < top or y > bottom):
+            # Coord is completely outside the rectangle area
+            # in that case, return one of the angles of the bounding rect
+            nearest_x = (coord.x() - width if x > right else coord.x())
+            nearest_y = (coord.y() if y < top else coord.y() - height)
+        elif x >= left and x <= right:
+            # x is within the rectangle
+            nearest_x = 0
+            nearest_y = (coord.y() if abs(y - top) < abs(y - bottom)
+                                   else coord.y() - height)
+        elif y >= top and y <= bottom:
+            # y is within the rectangle (lower priority than x)
+            nearest_x = (coord.x() if abs(x - left) < abs(x - right)
+                                   else coord.x() - width)
+            nearest_y = 0
+        return nearest_x, nearest_y
+
     @property
     def terminal_symbol(self):
         ''' Way to determine if a symbol is terminal (useful for branches) '''
@@ -961,6 +983,9 @@ class Symbol(QObject, QGraphicsPathItem, object):
     def mouse_move(self, event):
         ''' Handle resizing of items - moving is handled in subclass '''
         self.updateConnectionPoints()
+        # If any, update movable end points of connections 
+        for point in self.movable_points:
+            point.edge.end_connection.update_position()
         if self.mode == 'Resize':
             # Define the resizing based on where item has been grabbed
             if self.grabber.resize_mode.endswith('right'):
@@ -983,7 +1008,6 @@ class Symbol(QObject, QGraphicsPathItem, object):
                 height = max(user_height, 15)
                 width = max(user_width, 30)
             self.resize_item(QRect(0, 0, width, height))
-            #self.grabber.display()
 
     def mouse_release(self, _):
         ''' Default action when mouse is released: reset mode '''
