@@ -66,13 +66,11 @@ class Expression(object):
 
     def is_raw(self):
         ''' Helper function - check if an expression is a terminal primary '''
-        return (self.kind == 'primary' and self.var.kind != 'expression'
+        return (self.kind == 'primary'
+                and self.var.kind not in ('expression', 'constant')
                 and (self.var.kind != 'primaryId' or
                     (len(self.var.primaryId) < 2
                      and self.exprType['Kind'] != 'ReferenceType')))
-#       return (self.kind == 'primary' and self.var.kind != 'expression'
-#               and len(self.var.primaryId) < 2
-#               and self.exprType['Kind'] != 'ReferenceType')
 
 
 class Primary(object):
@@ -87,7 +85,8 @@ class Primary(object):
         # kind can be 'primaryId', 'enumeratedValue', 'numericalValue',
         # 'booleanValue', 'expression', 'ifThenElse', 'stringLiteral',
         # 'mantissaBaseExpFloat','emptyString' (SEQUENCE OF type value {}),
-        # 'choiceItem', 'sequenceOf', 'sequence', 'choiceDeterminant'
+        # 'choiceItem', 'sequenceOf', 'sequence', 'choiceDeterminant',
+        # 'constant'
         self.kind = None
         # primType is the ASN.1 type of the primary
         self.primType = None
@@ -121,8 +120,6 @@ class Primary(object):
         self.sequenceOf = []
         # sequence is a dict : { field1: Expression, field2: Expression }
         self.sequence = {}
-        # Hint for code generators: intermediate storage identifier
-        self.tmpVar = -1
 
     def __repr__(self):
         ''' Debug output for a primary '''
@@ -539,7 +536,7 @@ class Process(object):
         # global variables can be used to inherit variables
         self.global_variables = {}
 
-        # dataView: complete AST of the ASN.1 types
+        # dataview: complete AST of the ASN.1 types
         self.asn1Modules = None
         self.dataview = None
 
@@ -549,27 +546,18 @@ class Process(object):
         self.input_signals = []
         self.output_signals = []
 
-        # start, states, and terminators are useful for rendering backends
-#        self.start = None
-#        self.states = []
+        # terminators are useful for rendering backends
         self.terminators = []
 
         # Keep a list of labels and floating labels - useful for backends
         # List of Label
         self.labels = []
-        # List of Floating_label
-#        self.floating_labels = []
 
         # list of operators (not supported) and procedures
         self.operators = {}
         # list of Procedure (external procedures)
         self.procedures = []
-        # User-defined procedures: list of ogAST.Procedure
-#        self.inner_procedures = []
 
-        # list of type TextArea. Useful for rendering only
-        # use 'variables', 'operators' and 'procedures' to get useful content
-#        self.textAreas = []
 
         # The Mapping structure should be used for code generation backends
         # dictionnary: {'stateName': [class Input1, Input2,...], ...}
@@ -578,10 +566,13 @@ class Process(object):
 
         # list of type Transition - use 'mapping' to map index to inputs/states
         self.transitions = []
-        
+
         # Set of symbols contained in the process (type Automaton)
         self.content = Automaton(parent=self)
 
+        # List of timers (strings) declared in the process
+        self.timers = []
+
 
 class Block(object):
     ''' AST for a BLOCK entity '''
@@ -648,6 +639,8 @@ class AST(object):
         # Refs to the ASN.1 dataview AST (set with USE clauses)
         self.dataview = None
         self.asn1Modules = None
+        # Constants stored in the ASN.1 modules (type is unknown/unchecked)
+        self.asn1_constants = []
 
         # List of System
         self.systems = []
