@@ -43,11 +43,12 @@ SDL_BLACKBOLD = ['\\b{word}\\b'.format(word=word) for word in (
                 'DCL', 'CALL', 'ELSE', 'IF', 'THEN', 'MANTISSA', 'BASE',
                 'EXPONENT', 'TRUE', 'FALSE', 'MOD', 'FI', 'WRITE', 'WRITELN',
                 'LENGTH', 'PRESENT', 'FPAR', 'TODO', 'FIXME', 'XXX',
-                'CHECKME', 'PROCEDURE', 'EXTERNAL', 'IN', 'OUT')]
+                'CHECKME', 'PROCEDURE', 'EXTERNAL', 'IN', 'OUT', 'TIMER',
+                'SET_TIMER', 'RESET_TIMER')]
 
 SDL_REDBOLD = ['\\b{word}\\b'.format(word=word) for word in (
               'INPUT', 'OUTPUT', 'STATE', 'DECISION', 'NEXTSTATE',
-              'TASK', 'TIMER', 'PROCESS', 'LABEL', 'JOIN', 'CONNECTION')]
+              'TASK', 'PROCESS', 'LABEL', 'JOIN', 'CONNECTION')]
 
 
 # pylint: disable=R0904
@@ -202,7 +203,7 @@ class Decision(VerticalSymbol, object):
     blackbold = SDL_BLACKBOLD + ['\\b{}\\b'.format(word)
                                    for word in ('AND', 'OR')]
     redbold = SDL_REDBOLD
-    completion_list = set()
+    completion_list = {'length', 'present'}
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Decision()
@@ -693,9 +694,10 @@ class ProcedureCall(VerticalSymbol, object):
     _terminal_followers = ['Join', 'State', 'ProcedureStop']
     common_name = 'procedure_call'
     # Define reserved keywords for the syntax highlighter
-    blackbold = SDL_BLACKBOLD
+    blackbold = ['\\bWRITELN\\b', '\\bWRITE\\b',
+                 '\\bSET_TIMER\\b', '\\bRESET_TIMER\\b']
     redbold = SDL_REDBOLD
-    completion_list = set()
+    completion_list = {'set_timer', 'reset_timer', 'write', 'writeln'}
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Output(defName='callMe')
